@@ -2,10 +2,10 @@
 *    The PMW Music Typesetter - 3rd incarnation  *
 *************************************************/
 
-/* Copyright (c) Philip Hazel, 1991 - 2008 */
+/* Copyright (c) Philip Hazel, 1991 - 2013 */
 
 /* Written by Philip Hazel, starting November 1991 */
-/* This file last modified: September 2008 */
+/* This file last modified: March 2013 */
 
 
 /* This file contains the main code typesetting one note or chord */
@@ -295,7 +295,7 @@ else
   
   /* Set the horizontal position - centred rests and (semi)breves are a special
   case */
-
+  
   if ((n_flags & nf_centre) != 0)
     {
     int leftx = 4*main_stavemagn;
@@ -343,17 +343,20 @@ else
 
     else if ((n_flags & nf_hidden) == 0)  /* No number if Q-type rests */
       {
-      int font = curmovt->font_longrest;
       uschar s[8];
+      int font = curmovt->font_longrest;
       int size = (((curmovt->fontsizes)->fontsize_restct)*main_stavemagn)/1000;
       int *matrix = (curmovt->fontsizes)->fontmatrix_restct;
+      int i = (curmovt->codemultirests && out_manyrest < 9)? out_manyrest : 0; 
        
+      n_x = (leftx + out_barlinex - 
+        mac_muldiv(longrest_widths[i], main_stavemagn, 1000))/2;
+    
       if (matrix != NULL) memcpy(font_transform, matrix, 4*sizeof(int));
-       
-      n_x = (leftx + out_barlinex)/2 - 15*main_stavemagn;
+ 
       sprintf(CS s, "%d", out_manyrest);
-      out_string(s, font, size, n_x + 15*main_stavemagn -
-        font_stringwidth(s, font, size)/2 + out_Xadjustment,
+      out_string(s, font, size, out_Xadjustment + 
+        (leftx + out_barlinex - font_stringwidth(s, font, size))/2,
           out_ystave - 18*main_stavemagn - out_Yadjustment, 0);
       font_reset();
       }
