@@ -2,10 +2,10 @@
 *    The PMW Music Typesetter - 3rd incarnation  *
 *************************************************/
 
-/* Copyright (c) Philip Hazel, 1991 - 2012 */
+/* Copyright (c) Philip Hazel, 1991 - 2013 */
 
 /* Written by Philip Hazel, starting November 1991 */
-/* This file last modified: March 2012 */
+/* This file last modified: January 2013 */
 
 
 /* This file contains the code for generating an image of one individual note
@@ -27,6 +27,10 @@ static uschar common[] = {
 static uschar *reststrings[] = {
   US"*", US"+", US",", US"-", US".", US"z.w{{y.",
   US"zzx.w{{y.w{{y.", US"zzzx.w{{y.w{{y.w{{y." };
+  
+static uschar *multireststrings[] = {  /* Start for 2, last is for 8 */
+  US"*", US"*z+", US"*{{w*", US"*{{w*xz+", US"*{{w*xz*", 
+  US"*{{w*xz*z+", US"*{{w*xz*{{w*" };  
 
 static uschar *tailstrings[] = {
   US"",  US"",       US"",            US"",
@@ -609,6 +613,7 @@ show_rest(int x, int notetype)
 {
 int fontsize = (n_fontsize*main_stavemagn)/1000;
 int yoffset = n_restlevel;
+int y;
 
 /* Rests longer than a crotchet have to have ledger lines when they are printed
 off the stave. Also move a semibreve rest down on a 1-line stave and up on a
@@ -649,12 +654,19 @@ if (notetype <= minim)
 
 else yoffset += 4000 + n_pcorrection;
 
-if (notetype >= 0)
-  ps_musstring(reststrings[n_notetype], fontsize,
-    x, out_ystave - (yoffset*main_stavemagn)/1000);
+y = out_ystave - (yoffset*main_stavemagn)/1000;
 
-else ps_muschar(x, out_ystave - (yoffset*main_stavemagn)/1000,
-  mc_longrest, fontsize);
+if (notetype >= 0)
+  ps_musstring(reststrings[n_notetype], fontsize, x, y);
+
+else if (curmovt->codemultirests && out_manyrest < 9)
+  {
+ 
+  ps_musstring(multireststrings[out_manyrest - 2], fontsize, x, y - 2000);
+
+  } 
+
+else ps_muschar(x, y, mc_longrest, fontsize);
 }
 
 
@@ -1157,7 +1169,8 @@ if (n_ornament != NULL)
         }
       break;
 
-      /* Tremolos are handled with their own code */
+      /* Tremolos are handled with their own code. Vertical movement is
+      permitted. */
 
       case or_trem3:
       *p++ = 146;
@@ -1174,7 +1187,8 @@ if (n_ornament != NULL)
       if (n_upflag)
         {
         int yy = (n_ornament->ornament == or_trem3)? 4000 : 2000;
-        y = (y + ((n_ornament->ornament == or_trem1)? 4 : 2))* 1000;
+        y += (n_ornament->ornament == or_trem1)? 4 : 2;
+        y = y*1000 + n_ornament->y;
         if (out_beaming && n_stemlength >= yy) y += (n_stemlength - yy)/2;
         if (main_righttoleft) x -= 5000;
         }
@@ -1187,7 +1201,7 @@ if (n_ornament != NULL)
           case or_trem2: y -= 14; break;
           case or_trem1: y -= (y & 2)? 10 : 12; break;
           }
-        y = y*1000;
+        y = y*1000 + n_ornament->y;
         if (out_beaming && n_stemlength >= yy) y -= (n_stemlength - yy)/2;
         if (main_righttoleft) x += 5000;
         }
